@@ -33,6 +33,7 @@ class Walker(base.Walker):
 
   def _build(self, initializer=None):
     self._end_effectors_pos_sensors = []
+    self._obs_on_other = {}
     try:
       self._initializers = tuple(initializer)
     except TypeError:
@@ -66,7 +67,8 @@ class Walker(base.Walker):
     """
     return 0.
 
-  @abc.abstractproperty
+  @property
+  @abc.abstractmethod
   def ground_contact_geoms(self):
     """Geoms in this walker that are expected to be in contact with the ground.
 
@@ -101,6 +103,10 @@ class Walker(base.Walker):
   def body_geom_ids(self):
     return self._body_geom_ids
 
+  @property
+  def obs_on_other(self):
+    return self._obs_on_other
+
   def end_effector_contacts(self, physics):
     """Collect the contacts with the end effectors.
 
@@ -147,11 +153,13 @@ class Walker(base.Walker):
                                                          contact.geom2), 0.))
     return contacts
 
-  @abc.abstractproperty
+  @property
+  @abc.abstractmethod
   def end_effectors(self):
     raise NotImplementedError
 
-  @abc.abstractproperty
+  @property
+  @abc.abstractmethod
   def egocentric_camera(self):
     raise NotImplementedError
 
